@@ -2,7 +2,7 @@
 # vim: ts=4:sw=4:expandtab
 
 # BleachBit
-# Copyright (C) 2008-2023 Andrew Ziem
+# Copyright (C) 2008-2025 Andrew Ziem
 # https://www.bleachbit.org
 #
 # This program is free software: you can redistribute it and/or modify
@@ -34,7 +34,7 @@ if 'posix' == os.name:
 
     # The two imports from bleachbit must come after sys.path.append(..)
     import bleachbit.Unix
-    from bleachbit import _
+    from bleachbit.Language import get_text as _
 
     if (
         bleachbit.Unix.is_display_protocol_wayland_and_root_not_allowed()
@@ -47,13 +47,15 @@ if 'posix' == os.name:
 if os.name == 'nt':
     # change error handling to avoid popup with GTK 3
     # https://github.com/bleachbit/bleachbit/issues/651
-    import win32api
-    import win32con
+    import win32api  # pylint: disable=import-error
+    import win32con  # pylint: disable=import-error
     win32api.SetErrorMode(win32con.SEM_FAILCRITICALERRORS |
                           win32con.SEM_NOGPFAULTERRORBOX | win32con.SEM_NOOPENFILEERRORBOX)
 
 if 1 == len(sys.argv):
-    import bleachbit.GUI
+    # Import GUI inside the condition for Linux packagers to
+    # separate GUI into another package.
+    import bleachbit.GUI  # pylint: disable=ungrouped-imports
     app = bleachbit.GUI.Bleachbit()
     sys.exit(app.run(sys.argv))
 else:
