@@ -1,9 +1,8 @@
 # vim: ts=4:sw=4:expandtab
 # -*- coding: UTF-8 -*-
 
-
 # BleachBit
-# Copyright (C) 2008-2020 Andrew Ziem
+# Copyright (C) 2008-2025 Andrew Ziem
 # https://www.bleachbit.org
 #
 # This program is free software: you can redistribute it and/or modify
@@ -24,21 +23,15 @@
 Preferences dialog
 """
 
-from __future__ import absolute_import
-
-from bleachbit import _, _p, online_update_notification_enabled
+from bleachbit import online_update_notification_enabled
 from bleachbit.Options import options
 from bleachbit import GuiBasic
+from bleachbit.Language import get_active_language_code, get_supported_language_code_name_dict, setup_translation
+from bleachbit.Language import get_text as _, pget_text as _p
 
 from gi.repository import Gtk
 import logging
 import os
-import sys
-
-if 'nt' == os.name:
-    from bleachbit import Windows
-if 'posix' == os.name:
-    from bleachbit import Unix
 
 logger = logging.getLogger(__name__)
 
@@ -50,8 +43,9 @@ class PreferencesDialog:
 
     """Present the preferences dialog and save changes"""
 
-    def __init__(self, parent, cb_refresh_operations):
+    def __init__(self, parent, cb_refresh_operations, cb_set_windows10_theme):
         self.cb_refresh_operations = cb_refresh_operations
+        self.cb_set_windows10_theme = cb_set_windows10_theme
 
         self.parent = parent
         self.dialog = Gtk.Dialog(title=_("Preferences"),
@@ -96,49 +90,126 @@ class PreferencesDialog:
         if 'auto_hide' == path:
             self.refresh_operations = True
         if 'dark_mode' == path:
+            if 'nt' == os.name and options.get('win10_theme'):
+                self.cb_set_windows10_theme()
             Gtk.Settings.get_default().set_property(
                 'gtk-application-prefer-dark-theme', options.get('dark_mode'))
+        if 'win10_theme' == path:
+            self.cb_set_windows10_theme()
         if 'debug' == path:
             from bleachbit.Log import set_root_log_level
-            set_root_log_level()
+            set_root_log_level(options.get('debug'))
+        if 'kde_shred_menu_option' == path:
+            from bleachbit.DesktopMenuOptions import install_kde_service_menu_file
+            install_kde_service_menu_file()
 
-    def __general_page(self):
-        """Return a widget containing the general page"""
+    def __create_update_widgets(self, vbox):
+        """Create and configure update-related checkboxes."""
+        if not online_update_notification_enabled:
+            return
+        cb_updates = Gtk.CheckButton.new_with_label(
+            _("Check periodically for software updates via the Internet"))
+        cb_updates.set_active(options.get('check_online_updates'))
+        cb_updates.connect(
+            'toggled', self.__toggle_callback, 'check_online_updates')
+        cb_updates.set_tooltip_text(
+            _("If an update is found, you will be given the option to view information about it.  Then, you may manually download and install the update."))
+        vbox.pack_start(cb_updates, False, True, 0)
 
-        vbox = Gtk.Box(orientation=Gtk.Orientation.VERTICAL)
+        updates_box = Gtk.Box(orientation=Gtk.Orientation.VERTICAL)
+        updates_box.set_border_width(10)
 
-        if online_update_notification_enabled:
-            cb_updates = Gtk.CheckButton.new_with_label(
-                _("Check periodically for software updates via the Internet"))
-            cb_updates.set_active(options.get('check_online_updates'))
-            cb_updates.connect(
-                'toggled', self.__toggle_callback, 'check_online_updates')
-            cb_updates.set_tooltip_text(
-                _("If an update is found, you will be given the option to view information about it.  Then, you may manually download and install the update."))
-            vbox.pack_start(cb_updates, False, True, 0)
+        self.cb_beta = Gtk.CheckButton.new_with_label(
+            label=_("Check for new beta releases"))
+        self.cb_beta.set_active(options.get('check_beta'))
+        self.cb_beta.set_sensitive(options.get('check_online_updates'))
+        self.cb_beta.connect(
+            'toggled', self.__toggle_callback, 'check_beta')
+        updates_box.pack_start(self.cb_beta, False, True, 0)
 
-            updates_box = Gtk.Box(orientation=Gtk.Orientation.VERTICAL)
-            updates_box.set_border_width(10)
+        if 'nt' == os.name:
+            self.cb_winapp2 = Gtk.CheckButton.new_with_label(
+                _("Download and update cleaners from community (winapp2.ini)"))
+            self.cb_winapp2.set_active(options.get('update_winapp2'))
+            self.cb_winapp2.set_sensitive(
+                options.get('check_online_updates'))
+            self.cb_winapp2.connect(
+                'toggled', self.__toggle_callback, 'update_winapp2')
+            updates_box.pack_start(self.cb_winapp2, False, True, 0)
+        vbox.pack_start(updates_box, False, True, 0)
 
-            self.cb_beta = Gtk.CheckButton.new_with_label(
-                label=_("Check for new beta releases"))
-            self.cb_beta.set_active(options.get('check_beta'))
-            self.cb_beta.set_sensitive(options.get('check_online_updates'))
-            self.cb_beta.connect(
-                'toggled', self.__toggle_callback, 'check_beta')
-            updates_box.pack_start(self.cb_beta, False, True, 0)
+    def __create_language_widgets(self, vbox):
+        """Create and configure language selection widgets."""
+        lang_box = Gtk.Box(orientation=Gtk.Orientation.VERTICAL)
+        self.cb_auto_lang = Gtk.CheckButton(
+            label=_("Auto-detect language"))
+        is_auto_detect = options.get("auto_detect_lang")
+        self.cb_auto_lang.set_active(is_auto_detect)
+        self.cb_auto_lang.set_tooltip_text(
+            _("Automatically detect the system language"))
+        lang_box.pack_start(self.cb_auto_lang, False, True, 0)
 
-            if 'nt' == os.name:
-                self.cb_winapp2 = Gtk.CheckButton.new_with_label(
-                    _("Download and update cleaners from community (winapp2.ini)"))
-                self.cb_winapp2.set_active(options.get('update_winapp2'))
-                self.cb_winapp2.set_sensitive(
-                    options.get('check_online_updates'))
-                self.cb_winapp2.connect(
-                    'toggled', self.__toggle_callback, 'update_winapp2')
-                updates_box.pack_start(self.cb_winapp2, False, True, 0)
-            vbox.pack_start(updates_box, False, True, 0)
+        self.lang_select_box = Gtk.Box(orientation=Gtk.Orientation.HORIZONTAL)
+        lang_label = Gtk.Label(label=_("Language:"))
+        lang_label.set_margin_start(20)  # Add some indentation
+        self.lang_select_box.pack_start(lang_label, False, True, 5)
 
+        self.lang_combo = Gtk.ComboBoxText()
+        current_lang_code = get_active_language_code()
+        # Add available languages
+        lang_idx = 0
+        active_language_idx = None
+        try:
+            supported_langs = get_supported_language_code_name_dict().items()
+        except:
+            logger.error("Failed to get list of supported languages")
+            supported_langs = [('en_us', 'English')]
+        for lang_code, native in supported_langs:
+            if native:
+                self.lang_combo.append_text(f"{native} ({lang_code})")
+            else:
+                self.lang_combo.append_text(lang_code)
+            if lang_code == current_lang_code:
+                active_language_idx = lang_idx
+            lang_idx += 1
+        if active_language_idx is not None:
+            self.lang_combo.set_active(active_language_idx)
+        # set_wrap_width() prevents infinite space to scroll up.
+        # https://github.com/bleachbit/bleachbit/issues/1764
+        self.lang_combo.set_wrap_width(1)
+        self.lang_select_box.pack_start(self.lang_combo, False, True, 0)
+        lang_box.pack_start(self.lang_select_box, False, True, 0)
+
+        vbox.pack_start(lang_box, False, True, 0)
+        self.lang_select_box.set_sensitive(not is_auto_detect)
+        self.cb_auto_lang.connect('toggled', self.on_auto_detect_toggled)
+        self.lang_combo.connect('changed', self.on_lang_changed)
+
+    def on_lang_changed(self, widget):
+        """Callback for when the language combobox is changed."""
+        text = widget.get_active_text()
+        # Extract language code from the format "Native Name (lang_code)"
+        lang_code = text.split("(")[-1].rstrip(")")
+        if lang_code:
+            options.set("forced_language", lang_code, section="bleachbit")
+        else:
+            logger.warning(
+                "No language code found in combobox for text %s", text)
+        setup_translation()
+        self.refresh_operations = True
+
+    def on_auto_detect_toggled(self, widget):
+        """Callback for when the auto-detect language checkbox is toggled."""
+        self.__toggle_callback(None, 'auto_detect_lang')
+        is_auto_detect = options.get("auto_detect_lang")
+        self.lang_select_box.set_sensitive(not is_auto_detect)
+        if is_auto_detect:
+            options.set("forced_language", "", section="bleachbit")
+        setup_translation()
+        self.refresh_operations = True
+
+    def __create_general_checkboxes(self, vbox):
+        """Create and configure general checkboxes."""
         # TRANSLATORS: This means to hide cleaners which would do
         # nothing.  For example, if Firefox were never used on
         # this system, this option would hide Firefox to simplify
@@ -153,7 +224,7 @@ class PreferencesDialog:
         # to prevent recovery of the data. You could also translate
         # 'Shred files to prevent recovery.'
         cb_shred = Gtk.CheckButton(
-            _("Overwrite contents of files to prevent recovery"))
+            label=_("Overwrite contents of files to prevent recovery"))
         cb_shred.set_active(options.get('shred'))
         cb_shred.connect('toggled', self.__toggle_callback, 'shred')
         cb_shred.set_tooltip_text(
@@ -176,24 +247,60 @@ class PreferencesDialog:
 
         # Use base 1000 over 1024?
         cb_units_iec = Gtk.CheckButton(
-            _("Use IEC sizes (1 KiB = 1024 bytes) instead of SI (1 kB = 1000 bytes)"))
+            label=_("Use IEC sizes (1 KiB = 1024 bytes) instead of SI (1 kB = 1000 bytes)"))
         cb_units_iec.set_active(options.get("units_iec"))
         cb_units_iec.connect('toggled', self.__toggle_callback, 'units_iec')
         vbox.pack_start(cb_units_iec, False, True, 0)
 
-        # Dark theme
-        cb_dark_mode = Gtk.CheckButton(
-            _("Dark mode"))
-        cb_dark_mode.set_active(options.get("dark_mode"))
-        cb_dark_mode.connect('toggled', self.__toggle_callback, 'dark_mode')
-        vbox.pack_start(cb_dark_mode, False, True, 0)
+    def __general_page(self):
+        """Return a widget containing the general page"""
+
+        vbox = Gtk.Box(orientation=Gtk.Orientation.VERTICAL)
+
+        self.__create_update_widgets(vbox)
+
+        self.__create_general_checkboxes(vbox)
+
+        # Remember window geometry (position and size)
+        self.cb_geom = Gtk.CheckButton(label=_("Remember window geometry"))
+        self.cb_geom.set_active(options.get("remember_geometry"))
+        self.cb_geom.connect(
+            'toggled', self.__toggle_callback, 'remember_geometry')
+        vbox.pack_start(self.cb_geom, False, True, 0)
 
         # Debug logging
-        cb_debug = Gtk.CheckButton(_("Show debug messages"))
+        cb_debug = Gtk.CheckButton(label=_("Show debug messages"))
         cb_debug.set_active(options.get("debug"))
         cb_debug.connect('toggled', self.__toggle_callback, 'debug')
         vbox.pack_start(cb_debug, False, True, 0)
 
+        # KDE context menu shred option
+        if 'nt' != os.name:
+            cb_kde_shred_menu_option = Gtk.CheckButton(
+                label=_("Add the shred context menu to KDE Plasma"))
+            cb_kde_shred_menu_option.set_active(
+                options.get("kde_shred_menu_option"))
+            cb_kde_shred_menu_option.connect(
+                'toggled', self.__toggle_callback, 'kde_shred_menu_option')
+            vbox.pack_start(cb_kde_shred_menu_option, False, True, 0)
+
+        # Dark theme
+        self.cb_dark_mode = Gtk.CheckButton(label=_("Dark mode"))
+        self.cb_dark_mode.set_active(options.get("dark_mode"))
+        self.cb_dark_mode.connect(
+            'toggled', self.__toggle_callback, 'dark_mode')
+        vbox.pack_start(self.cb_dark_mode, False, True, 0)
+
+        if 'nt' == os.name:
+            # Windows 10 theme
+            cb_win10_theme = Gtk.CheckButton(_("Windows 10 theme"))
+            cb_win10_theme.set_active(options.get("win10_theme"))
+            cb_win10_theme.connect(
+                'toggled', self.__toggle_callback, 'win10_theme')
+            vbox.pack_start(cb_win10_theme, False, True, 0)
+
+        self.__create_language_widgets(vbox)
+
         return vbox
 
     def __drives_page(self):
@@ -280,7 +387,7 @@ class PreferencesDialog:
 
         # populate data
         liststore = Gtk.ListStore('gboolean', str, str)
-        for lang, native in sorted(Unix.Locales.native_locale_names.items()):
+        for lang, native in get_supported_language_code_name_dict().items():
             liststore.append([(options.get_language(lang)), lang, native])
 
         # create treeview
@@ -305,101 +412,76 @@ class PreferencesDialog:
 
         # finish
         swindow = Gtk.ScrolledWindow()
+        swindow.set_overlay_scrolling(False)
         swindow.set_policy(Gtk.PolicyType.AUTOMATIC, Gtk.PolicyType.AUTOMATIC)
         swindow.set_size_request(300, 200)
         swindow.add(treeview)
         vbox.pack_start(swindow, False, True, 0)
         return vbox
 
+    def _check_path_exists(self, pathname, page_type):
+        """Check if a path exists in either whitelist or custom lists
+        Returns True if path exists, False otherwise"""
+        whitelist_paths = options.get_whitelist_paths()
+        custom_paths = options.get_custom_paths()
+
+        # Check in whitelist
+        for path in whitelist_paths:
+            if pathname == path[1]:
+                msg = _("This path already exists in the whitelist.")
+                GuiBasic.message_dialog(self.dialog,
+                                        msg,
+                                        Gtk.MessageType.ERROR,
+                                        Gtk.ButtonsType.OK,
+                                        _('Error'))
+                return True
+
+        # Check in custom
+        for path in custom_paths:
+            if pathname == path[1]:
+                msg = _("This path already exists in the custom list.")
+                GuiBasic.message_dialog(self.dialog,
+                                        msg,
+                                        Gtk.MessageType.ERROR,
+                                        Gtk.ButtonsType.OK,
+                                        _('Error'))
+                return True
+
+        return False
+
+    def _add_path(self, pathname, path_type, page_type, liststore, pathnames):
+        """Common function to add a path to either whitelist or custom list"""
+        if self._check_path_exists(pathname, page_type):
+            return
+
+        type_str = _('File') if path_type == 'file' else _('Folder')
+        liststore.append([type_str, pathname])
+        pathnames.append([path_type, pathname])
+
+        if page_type == LOCATIONS_WHITELIST:
+            options.set_whitelist_paths(pathnames)
+        else:
+            options.set_custom_paths(pathnames)
+
+    def _remove_path(self, treeview, liststore, pathnames, page_type):
+        """Common function to remove a path from either whitelist or custom list"""
+        treeselection = treeview.get_selection()
+        (model, _iter) = treeselection.get_selected()
+        if None == _iter:
+            return
+        pathname = model[_iter][1]
+        liststore.remove(_iter)
+        for this_pathname in pathnames:
+            if this_pathname[1] == pathname:
+                pathnames.remove(this_pathname)
+                if page_type == LOCATIONS_WHITELIST:
+                    options.set_whitelist_paths(pathnames)
+                else:
+                    options.set_custom_paths(pathnames)
+
     def __locations_page(self, page_type):
         """Return a widget containing a list of files and folders"""
 
-        def add_whitelist_file_cb(button):
-            """Callback for adding a file"""
-            title = _("Choose a file")
-            pathname = GuiBasic.browse_file(self.parent, title)
-            if pathname:
-                for this_pathname in pathnames:
-                    if pathname == this_pathname[1]:
-                        logger.warning(
-                            "'%s' already exists in whitelist", pathname)
-                        return
-                liststore.append([_('File'), pathname])
-                pathnames.append(['file', pathname])
-                options.set_whitelist_paths(pathnames)
-
-        def add_whitelist_folder_cb(button):
-            """Callback for adding a folder"""
-            title = _("Choose a folder")
-            pathname = GuiBasic.browse_folder(self.parent, title,
-                                              multiple=False, stock_button=Gtk.STOCK_ADD)
-            if pathname:
-                for this_pathname in pathnames:
-                    if pathname == this_pathname[1]:
-                        logger.warning(
-                            "'%s' already exists in whitelist", pathname)
-                        return
-                liststore.append([_('Folder'), pathname])
-                pathnames.append(['folder', pathname])
-                options.set_whitelist_paths(pathnames)
-
-        def remove_whitelist_path_cb(button):
-            """Callback for removing a path"""
-            treeselection = treeview.get_selection()
-            (model, _iter) = treeselection.get_selected()
-            if None == _iter:
-                # nothing selected
-                return
-            pathname = model[_iter][1]
-            liststore.remove(_iter)
-            for this_pathname in pathnames:
-                if this_pathname[1] == pathname:
-                    pathnames.remove(this_pathname)
-                    options.set_whitelist_paths(pathnames)
-
-        def add_custom_file_cb(button):
-            """Callback for adding a file"""
-            title = _("Choose a file")
-            pathname = GuiBasic.browse_file(self.parent, title)
-            if pathname:
-                for this_pathname in pathnames:
-                    if pathname == this_pathname[1]:
-                        logger.warning(
-                            "'%s' already exists in whitelist", pathname)
-                        return
-                liststore.append([_('File'), pathname])
-                pathnames.append(['file', pathname])
-                options.set_custom_paths(pathnames)
-
-        def add_custom_folder_cb(button):
-            """Callback for adding a folder"""
-            title = _("Choose a folder")
-            pathname = GuiBasic.browse_folder(self.parent, title,
-                                              multiple=False, stock_button=Gtk.STOCK_ADD)
-            if pathname:
-                for this_pathname in pathnames:
-                    if pathname == this_pathname[1]:
-                        logger.warning(
-                            "'%s' already exists in whitelist", pathname)
-                        return
-                liststore.append([_('Folder'), pathname])
-                pathnames.append(['folder', pathname])
-                options.set_custom_paths(pathnames)
-
-        def remove_custom_path_cb(button):
-            """Callback for removing a path"""
-            treeselection = treeview.get_selection()
-            (model, _iter) = treeselection.get_selected()
-            if None == _iter:
-                # nothing selected
-                return
-            pathname = model[_iter][1]
-            liststore.remove(_iter)
-            for this_pathname in pathnames:
-                if this_pathname[1] == pathname:
-                    pathnames.remove(this_pathname)
-                    options.set_custom_paths(pathnames)
-
         vbox = Gtk.Box(orientation=Gtk.Orientation.VERTICAL)
 
         # load data
@@ -424,7 +506,7 @@ class PreferencesDialog:
             # TRANSLATORS: "Paths" is used generically to refer to both files
             # and folders
             notice = Gtk.Label(
-                label=_("Theses paths will not be deleted or modified."))
+                label=_("These paths will not be deleted or modified."))
         elif LOCATIONS_CUSTOM == page_type:
             notice = Gtk.Label(
                 label=_("These locations can be selected for deletion."))
@@ -447,6 +529,7 @@ class PreferencesDialog:
 
         # finish tree view
         swindow = Gtk.ScrolledWindow()
+        swindow.set_overlay_scrolling(False)
         swindow.set_policy(Gtk.PolicyType.AUTOMATIC, Gtk.PolicyType.AUTOMATIC)
         swindow.set_size_request(300, 200)
         swindow.add(treeview)
@@ -454,25 +537,37 @@ class PreferencesDialog:
         vbox.pack_start(swindow, False, True, 0)
 
         # buttons that modify the list
+        def add_file_cb(button):
+            """Callback for adding a file"""
+            title = _("Choose a file")
+            pathname = GuiBasic.browse_file(self.parent, title)
+            if pathname:
+                self._add_path(pathname, 'file', page_type,
+                               liststore, pathnames)
+
+        def add_folder_cb(button):
+            """Callback for adding a folder"""
+            title = _("Choose a folder")
+            pathname = GuiBasic.browse_folder(self.parent, title,
+                                              multiple=False, stock_button=Gtk.STOCK_ADD)
+            if pathname:
+                self._add_path(pathname, 'folder', page_type,
+                               liststore, pathnames)
+
+        def remove_path_cb(button):
+            """Callback for removing a path"""
+            self._remove_path(treeview, liststore, pathnames, page_type)
+
         button_add_file = Gtk.Button.new_with_label(
             label=_p('button', 'Add file'))
-        if LOCATIONS_WHITELIST == page_type:
-            button_add_file.connect("clicked", add_whitelist_file_cb)
-        elif LOCATIONS_CUSTOM == page_type:
-            button_add_file.connect("clicked", add_custom_file_cb)
+        button_add_file.connect("clicked", add_file_cb)
 
         button_add_folder = Gtk.Button.new_with_label(
             label=_p('button', 'Add folder'))
-        if LOCATIONS_WHITELIST == page_type:
-            button_add_folder.connect("clicked", add_whitelist_folder_cb)
-        elif LOCATIONS_CUSTOM == page_type:
-            button_add_folder.connect("clicked", add_custom_folder_cb)
+        button_add_folder.connect("clicked", add_folder_cb)
 
         button_remove = Gtk.Button.new_with_label(label=_p('button', 'Remove'))
-        if LOCATIONS_WHITELIST == page_type:
-            button_remove.connect("clicked", remove_whitelist_path_cb)
-        elif LOCATIONS_CUSTOM == page_type:
-            button_remove.connect("clicked", remove_custom_path_cb)
+        button_remove.connect("clicked", remove_path_cb)
 
         button_box = Gtk.ButtonBox(orientation=Gtk.Orientation.HORIZONTAL)
         button_box.set_layout(Gtk.ButtonBoxStyle.START)
