@@ -2,7 +2,7 @@
 # -*- coding: UTF-8 -*-
 
 # BleachBit
-# Copyright (C) 2008-2020 Andrew Ziem
+# Copyright (C) 2008-2025 Andrew Ziem
 # https://www.bleachbit.org
 #
 # This program is free software: you can redistribute it and/or modify
@@ -23,16 +23,53 @@
 Test case for module FileUtilities
 """
 
-from __future__ import absolute_import, print_function
 
-from tests import common
-from bleachbit.FileUtilities import *
-from bleachbit.General import run_external, sudo_mode
+from bleachbit.FileUtilities import (
+    bytes_to_human,
+    children_in_directory,
+    clean_ini,
+    clean_json,
+    delete,
+    detect_encoding,
+    ego_owner,
+    exe_exists,
+    exists_in_path,
+    expand_glob_join,
+    extended_path_undo,
+    extended_path,
+    free_space,
+    get_filesystem_type,
+    getsize,
+    getsizedir,
+    globex,
+    guess_overwrite_paths,
+    human_to_bytes,
+    is_dir_empty,
+    listdir,
+    open_files_lsof,
+    OpenFiles,
+    same_partition,
+    uris_to_paths,
+    vacuum_sqlite3,
+    whitelisted,
+    wipe_contents,
+    wipe_name,
+    wipe_path
+)
+from bleachbit.General import gc_collect, run_external
 from bleachbit.Options import options
-from bleachbit import expanduser, expandvars, logger
+from bleachbit import logger
+from tests import common
 
 import json
+import locale
+import os
+import random
+import re
+import subprocess
 import sys
+import tempfile
+import time
 import unittest
 
 
@@ -41,8 +78,6 @@ def test_ini_helper(self, execute):
 
     teststrings = [
         b'#Test\n[RecentsMRL]\nlist=C:\\Users\\me\\Videos\\movie.mpg,C:\\Users\\me\\movie2.mpg\n\n',
-        u'#Test\n[RecentsMRL]\nlist=C:\\Юзъри\\me\\Videos\\movie.mpg,C:\\Users\\me\\movie2.mpg\n\n'.encode(
-            'ISO-8859-5')
     ]
     for teststr in teststrings:
 
@@ -52,19 +87,21 @@ def test_ini_helper(self, execute):
         size = os.path.getsize(filename)
         self.assertEqual(len(teststr), size)
 
-        # section does not exist
+        # The section does not exist, so no change.
         execute(filename, 'Recents', None)
         self.assertEqual(len(teststr), os.path.getsize(filename))
 
-        # parameter does not exist
+        # The parameter does not exist, so no change.
         execute(filename, 'RecentsMRL', 'files')
         self.assertEqual(len(teststr), os.path.getsize(filename))
 
-        # parameter does exist
+        # The parameter does exist, so the file shrinks.
+        # The file will be size 14 if chardet is available.
+        # Otherwise, size will be 17 with BOM.
         execute(filename, 'RecentsMRL', 'list')
-        self.assertEqual(14, os.path.getsize(filename))
+        self.assertIn(os.path.getsize(filename), (14,17))
 
-        # section does exist
+        # The section does exist, so the file shrinks.
         execute(filename, 'RecentsMRL', None)
         self.assertEqual(0, os.path.getsize(filename))
 
@@ -85,7 +122,7 @@ def test_json_helper(self, execute):
     # create test file
     (fd, filename) = tempfile.mkstemp(
         prefix='bleachbit-test-json', dir=self.tempdir)
-    os.write(fd, '{ "deleteme" : 1, "spareme" : { "deletemetoo" : 1 } }')
+    os.write(fd, b'{ "deleteme" : 1, "spareme" : { "deletemetoo" : 1 } }')
     os.close(fd)
     self.assertExists(filename)
     self.assertEqual(load_js(filename), expected)
@@ -118,16 +155,46 @@ def test_json_helper(self, execute):
 class FileUtilitiesTestCase(common.BleachbitTestCase):
     """Test case for module FileUtilities"""
 
-    def test_bytes_to_human(self):
-        """Unit test for class bytes_to_human"""
-
-        old_locale = locale.getlocale(locale.LC_NUMERIC)
+    def setUp(self):
+        """Call before each test method"""
+        super().setUp()
+        self.old_locale = locale.getlocale(locale.LC_NUMERIC)
         locale.setlocale(locale.LC_NUMERIC, 'C')
 
-        # test one-way conversion for predefined values
-        # each test is a tuple in the format: (bytes, SI, EIC)
+    def tearDown(self):
+        """Call after each test method"""
+        if self.old_locale == (None, None):
+            locale.setlocale(locale.LC_NUMERIC, 'C')
+            return
+        try:
+            if self.old_locale[0] is None:
+                locale.setlocale(locale.LC_NUMERIC, None)
+            else:
+                try:
+                    # First try with the full locale string
+                    locale.setlocale(locale.LC_NUMERIC, '.'.join(
+                        filter(None, self.old_locale)))
+                except locale.Error:
+                    # If that fails, try just the language code part
+                    try:
+                        locale.setlocale(locale.LC_NUMERIC, self.old_locale[0])
+                    except locale.Error as e:
+                        print(
+                            f'Failed to restore locale with just language code {self.old_locale[0]}: {e}')
+        except locale.Error as e:
+            print(f'Failed to restore locale {self.old_locale}: {e}')
+        # Check that running getlocale again does not raise an exception.
+        # For me, getlocale() fails after successful setlocale(..., 'en_MX') on Windows.
+        locale.getlocale(locale.LC_NUMERIC)
+        self.assertEqual(locale.getlocale(locale.LC_NUMERIC), self.old_locale)
+
+    def test_bytes_to_human_one_way(self):
+        """Test one-way conversion of bytes_to_human()"""
+
+        # Test one-way conversion for predefined values
+        # Each test is a tuple in the format: (bytes, SI, EIC)
         tests = ((-1, '-1B', '-1B'),
-                 (0, '0', '0'),
+                 (0, '0B', '0B'),
                  (1, '1B', '1B'),
                  (1000, '1kB', '1000B'),
                  (1024, '1kB', '1KiB'),
@@ -155,33 +222,37 @@ class FileUtilitiesTestCase(common.BleachbitTestCase):
             self.assertEqual(test[1], si,
                              'bytes_to_human(%d) SI = %s but expected %s' % (test[0], si, test[1]))
 
-        # test roundtrip conversion for random values
-        import random
-        for n in range(0, 1000):
+
+    def test_bytes_to_human_roundtrip(self):
+        """Test roundtrip conversion of bytes_to_human()
+
+        Example: 1,964,950 -> 2MB -> 2,000,000 with difference of 1.78% (0.0178).
+        """
+
+        for _n in range(0, 1000):
             bytes1 = random.randrange(0, 1000 ** 4)
             human = bytes_to_human(bytes1)
             bytes2 = human_to_bytes(human)
             error = abs(float(bytes2 - bytes1) / bytes1)
-            self.assertLess(abs(error), 0.01, "%d (%s) is %.2f%% different than %d" %
-                            (bytes1, human, error * 100, bytes2))
+            self.assertLess(abs(
+                error), 0.02, f"{bytes1:,} ({human}) is {error * 100:.2f}% different than {bytes2:,}")
 
-        # test localization
-        if hasattr(locale, 'format_string'):
-            try:
-                locale.setlocale(locale.LC_NUMERIC, 'de_DE.utf8')
-            except:
-                logger.warning('exception when setlocale to de_DE.utf8')
-            else:
-                self.assertEqual("1,01GB", bytes_to_human(1000 ** 3 + 5812389))
-
-        # clean up
-        locale.setlocale(locale.LC_NUMERIC, old_locale)
+    def test_bytes_to_human_localization(self):
+        """Test localization of bytes_to_human()"""
+        if not hasattr(locale, 'format_string'):
+            self.skipTest('Locale module does not support format_string')
+        try:
+            locale.setlocale(locale.LC_NUMERIC, 'de_DE.utf8')
+        except:
+            logger.warning('exception when setlocale to de_DE.utf8')
+        else:
+            self.assertEqual("1,01GB", bytes_to_human(1000 ** 3 + 5812389))
 
     def test_children_in_directory(self):
         """Unit test for function children_in_directory()"""
 
         # test an existing directory that usually exists
-        dirname = expanduser("~/.config")
+        dirname = os.path.expanduser("~/.config")
         for filename in children_in_directory(dirname, True):
             self.assertTrue(os.path.isabs(filename))
         for filename in children_in_directory(dirname, False):
@@ -239,24 +310,13 @@ class FileUtilitiesTestCase(common.BleachbitTestCase):
         delete('does-not-exist', ignore_missing=True)
         self.assertRaises(OSError, delete, 'does-not-exist')
 
-    def test_delete_not_empty(self):
-        """Test for scenario directory is not empty"""
-        print('hello')
-        fn = os.path.join(self.tempdir, 'make-directory-not-empty')
-        common.touch_file(fn)
-        self.assertExists(fn)
-        self.assertExists(self.tempdir)
-        delete(self.tempdir, allow_shred=False)
-        self.assertExists(fn)
-        self.assertExists(self.tempdir)
-
     def delete_helper(self, shred):
         """Called by test_delete() with shred = False and = True"""
 
         # test deleting with various kinds of filenames
-        hebrew = u"עִבְרִית"
-        katanana = u"アメリカ"
-        umlauts = u"ÄäǞǟËëḦḧÏïḮḯÖöȪȫṎṏT̈ẗÜüǕǖǗǘǙǚǛǜṲṳṺṻẄẅẌẍŸÿ"
+        hebrew = "עִבְרִית"
+        katanana = "アメリカ"
+        umlauts = "ÄäǞǟËëḦḧÏïḮḯÖöȪȫṎṏT̈ẗÜüǕǖǗǘǙǚǛǜṲṳṺṻẄẅẌẍŸÿ"
 
         tests = ['.prefixandsuffix',  # simple
                  "x".zfill(150),  # long
@@ -264,9 +324,9 @@ class FileUtilitiesTestCase(common.BleachbitTestCase):
                  "''",  # quotation mark
                  "~`!@#$%^&()-_+=x",  # non-alphanumeric characters
                  "[]{};'.,x",  # non-alphanumeric characters
-                 u'abcdefgh',  # simple Unicode
-                 u'J\xf8rgen Scandinavian',
-                 u'\u2014em-dash',  # LP#1454030
+                 'abcdefgh',  # simple Unicode
+                 'J\xf8rgen Scandinavian',
+                 '\u2014em-dash',  # LP#1454030
                  hebrew,
                  katanana,
                  umlauts,
@@ -277,7 +337,7 @@ class FileUtilitiesTestCase(common.BleachbitTestCase):
                       ' ', '.file.']  # Windows filenames cannot end with space or period
         for test in tests:
             # create the file
-            filename = self.write_file(test, "top secret")
+            filename = self.write_file(test, b"top secret")
             # delete the file
             delete(filename, shred)
             self.assertNotExists(filename)
@@ -334,30 +394,24 @@ class FileUtilitiesTestCase(common.BleachbitTestCase):
             self.assertNotExists(linkname)
             self.assertNotLExists(linkname)
 
-        windows_vista_or_newer = False
         if 'nt' == os.name:
-            from bleachbit.Windows import parse_windows_build
-            # Windows Vista = 6.0
-            windows_vista_or_newer = parse_windows_build() >= 6.0
-
-        if windows_vista_or_newer:
             logger.debug('testing symbolic link')
             import ctypes
             kern = ctypes.windll.LoadLibrary("kernel32.dll")
 
             def win_symlink(src, linkname):
-                rc = kern.CreateSymbolicLinkA(linkname, src, 0)
+                rc = kern.CreateSymbolicLinkW(linkname, src, 0)
                 if rc == 0:
-                    print('CreateSymbolicLinkA(%s, %s)' % (linkname, src))
-                    print('CreateSymolicLinkA() failed, error = %s' %
+                    print('CreateSymbolicLinkW(%s, %s)' % (linkname, src))
+                    print('CreateSymolicLinkW() failed, error = %s' %
                           ctypes.FormatError())
                     self.assertNotEqual(rc, 0)
             symlink_helper(win_symlink)
 
-        # below this point, only posix
-        if 'nt' == os.name:
             return
 
+        # below this point, only posix
+
         # test file with mode 0444/-r--r--r--
         filename = self.write_file('bleachbit-test-0444')
         os.chmod(filename, 0o444)
@@ -381,17 +435,6 @@ class FileUtilitiesTestCase(common.BleachbitTestCase):
         delete(path, shred)
         self.assertNotExists(path)
 
-    def test_delete_read_only(self):
-        """Unit test for delete() with read-only file"""
-        for shred in (False, True):
-            fn = os.path.join(self.tempdir, 'read-only')
-            common.touch_file(fn)
-            import stat
-            os.chmod(fn, stat.S_IREAD)
-            self.assertExists(fn)
-            delete(fn, shred=shred)
-            self.assertNotExists(fn)
-
     @common.skipUnlessWindows
     def test_delete_hidden(self):
         """Unit test for delete() with hidden file"""
@@ -410,8 +453,9 @@ class FileUtilitiesTestCase(common.BleachbitTestCase):
         """Unit test for delete() with locked file"""
         # set up
         def test_delete_locked_setup():
-            (fd, filename) = tempfile.mkstemp(prefix='bleachbit-test-worker')
-            os.write(fd, '123')
+            (fd, filename) = tempfile.mkstemp(
+                prefix='bleachbit-test-fileutilities')
+            os.write(fd, b'123')
             os.close(fd)
             self.assertExists(filename)
             self.assertEqual(3, getsize(filename))
@@ -471,17 +515,78 @@ class FileUtilitiesTestCase(common.BleachbitTestCase):
         msg = 'error calling umount\nargs=%s\nstderr=%s' % (args, stderr)
         self.assertEqual(rc, 0, msg)
 
+    def test_delete_not_empty(self):
+        """Test for scenario directory is not empty"""
+        # common.py puts bleachbit.ini in self.tempdir, but it may
+        # not be flushed
+        dirname = os.path.join(self.tempdir, 'a_dir')
+        os.mkdir(dirname)
+        self.assertFalse(is_dir_empty(self.tempdir))
+        self.assertTrue(is_dir_empty(dirname))
+        fn = os.path.join(dirname, 'a_file')
+        common.touch_file(fn)
+        self.assertFalse(is_dir_empty(dirname))
+        self.assertExists(fn)
+        self.assertExists(dirname)
+        self.assertExists(self.tempdir)
+
+        # Make sure shredding does not leave a renamed directory like
+        # in https://github.com/bleachbit/bleachbit/issues/783
+        for allow_shred in (False, True):
+            delete(dirname, allow_shred=allow_shred)
+            self.assertExists(fn)
+            self.assertExists(dirname)
+        os.remove(fn)
+        self.assertTrue(is_dir_empty(dirname))
+
+    def test_delete_read_only(self):
+        """Unit test for delete() with read-only file"""
+        for shred in (False, True):
+            fn = os.path.join(self.tempdir, 'read-only')
+            common.touch_file(fn)
+            import stat
+            os.chmod(fn, stat.S_IREAD)
+            self.assertExists(fn)
+            delete(fn, shred=shred)
+            self.assertNotExists(fn)
+
+    def test_detect_encoding(self):
+        """Unit test for detect_encoding"""
+        eat_glass = '나는 유리를 먹을 수 있어요. 그래도 아프지 않아요'
+        bom = '\ufeff' + eat_glass  # Add BOM for utf-8-sig
+        tests = (('This is just an ASCII file', 'ascii'),
+                 (eat_glass, 'utf-8'),
+                 (eat_glass, 'EUC-KR'),
+                 (bom, 'UTF-8-SIG'))
+        for file_contents, expected_encoding in tests:
+            with self.subTest(encoding=expected_encoding):
+                with tempfile.NamedTemporaryFile(mode='w', delete=False, encoding=expected_encoding) as temp:
+                    temp.write(file_contents)
+                    temp.flush()
+                det = detect_encoding(temp.name)
+                self.assertEqual(det, expected_encoding, f"{file_contents} -> {det}, check that chardet is available")
+
     @common.skipIfWindows
     def test_ego_owner(self):
         """Unit test for ego_owner()"""
         self.assertEqual(ego_owner('/bin/ls'), os.getuid() == 0)
 
-    def test_exists_in_path(self):
-        """Unit test for exists_in_path()"""
-        filename = 'ls'
-        if 'nt' == os.name:
-            filename = 'cmd.exe'
-        self.assertTrue(exists_in_path(filename))
+    def test_execute_sqlite3(self):
+        """Unit test for execute_sqlite3()"""
+        from bleachbit.FileUtilities import execute_sqlite3
+        db_path = self.mkstemp(prefix='bleachbit-test-file', suffix='.sqlite')
+
+        cmds = ['CREATE TABLE test (id INTEGER PRIMARY KEY, name TEXT)',
+                "INSERT INTO test (name) VALUES ('A'), ('B')",
+                "UPDATE test SET name = 'C' WHERE name = 'B'",
+                "DELETE FROM test WHERE name = 'C'"]
+
+        execute_sqlite3(db_path, ';'.join(cmds))
+        execute_sqlite3(db_path, 'vacuum')
+
+        gc_collect()
+        os.unlink(db_path)
+        self.assertNotExists(db_path)
 
     def test_exe_exists(self):
         """Unit test for exe_exists()"""
@@ -497,17 +602,50 @@ class FileUtilitiesTestCase(common.BleachbitTestCase):
         for test in tests:
             self.assertEqual(exe_exists(test[0]), test[1])
 
+    def test_exists_in_path(self):
+        """Unit test for exists_in_path()"""
+        filename = 'ls'
+        if 'nt' == os.name:
+            filename = 'cmd.exe'
+        self.assertTrue(exists_in_path(filename))
+
     def test_expand_glob_join(self):
         """Unit test for expand_glob_join()"""
         if 'posix' == os.name:
             expand_glob_join('/bin', '*sh')
         if 'nt' == os.name:
-            expand_glob_join('c:\windows', '*.exe')
+            expand_glob_join(r'c:\windows', '*.exe')
 
-    def test_expandvars(self):
-        """Unit test for expandvars()."""
-        expanded = expandvars('$HOME')
-        self.assertIsUnicodeString(expanded)
+    def test_expand_vars_user(self):
+        """Unit test for expandvars() and expanduser()
+
+        We had custom implementations of expandvars() and expanduser()
+        until commit 3a3913 that switched to Python 3.
+        """
+
+        if 'nt' == os.name:
+            vars = ('%USERPROFILE%', '$userprofile', '${USERprofile}')
+        else:
+            vars = ['$HOME', '${HOME}']
+        for var in vars:
+            expanded = os.path.expandvars(var)
+            self.assertIsString(expanded)
+            self.assertNotEqual(
+                expanded, var, 'Environment variable was not expanded')
+            self.assertEqual(expanded, os.path.expanduser('~'))
+            self.assertExists(expanded)
+            # An absolute path should not be altered.
+            self.assertEqual(expanded, os.path.expandvars(expanded))
+            self.assertEqual(expanded, os.path.expanduser(expanded))
+        # An empty string should not be altered.
+        self.assertEqual(os.path.expandvars(''), '')
+
+        # Non-existent variables should not be altered.
+        self.assertEqual(os.path.expandvars('$nonexistent'), '$nonexistent')
+
+        # A relative path should not be expanded.
+        for p in ('common', 'common/', 'common', 'home', 'userprofile'):
+            self.assertEqual(os.path.expanduser(p), p)
 
     def test_extended_path(self):
         """Unit test for extended_path() and extended_path_undo()"""
@@ -531,38 +669,66 @@ class FileUtilitiesTestCase(common.BleachbitTestCase):
 
     def test_free_space(self):
         """Unit test for free_space()"""
-        home = expanduser('~')
+        home = os.path.expanduser('~')
         result = free_space(home)
         self.assertNotEqual(result, None)
         self.assertGreater(result, -1)
         self.assertIsInteger(result)
 
-        # compare to WMIC
-        if 'nt' != os.name:
-            return
-        args = ['wmic',  'LogicalDisk', 'get', 'DeviceID,', 'FreeSpace']
+    @common.skipUnlessWindows
+    def test_free_space_windows(self):
+        """Unit test for free_space() on Windows"""
+        args = ['wmic', 'LogicalDisk', 'get', 'DeviceID,', 'FreeSpace']
         (rc, stdout, stderr) = run_external(args)
         if rc:
             print('error calling WMIC\nargs=%s\nstderr=%s' % (args, stderr))
             return
-        import re
         for line in stdout.split('\n'):
             line = line.strip()
-            if not re.match('([A-Z]):\s+(\d+)', line):
+            if not re.match(r'([A-Z]):\s+(\d+)', line):
                 continue
-            drive, bytes_free = re.split('\s+', line)
-            print('Checking free space for %s' % drive)
+            drive, bytes_free = re.split(r'\s+', line)
             bytes_free = int(bytes_free)
-            free = free_space(unicode(drive))
+            free = free_space(drive)
             self.assertEqual(bytes_free, free)
 
+    def test_get_filesystem_type(self):
+        """Unit test for get_filesystem_type()"""
+        home = os.path.expanduser('~')
+        if os.name == 'nt':
+            path = self.tempdir
+            fs_type = get_filesystem_type(path)[0]
+            while path:
+                print(path)
+                self.assertEqual(get_filesystem_type(path)[0], fs_type)
+                self.assertEqual(get_filesystem_type(path.lower())[0], fs_type)
+                self.assertEqual(get_filesystem_type(path.upper())[0], fs_type)
+                if path == os.path.dirname(path):
+                    break
+                path = os.path.dirname(path)
+            self.assertNotEqual(fs_type, 'unknown')
+            self.assertEqual(get_filesystem_type(r'a:\\')[0], 'unknown')
+            self.assertEqual(get_filesystem_type(r'Z:\\')[0], 'unknown')
+            for check_path in (home, r'c:\\', r'c:'):
+                self.assertEqual(get_filesystem_type(
+                    check_path)[0], 'NTFS', check_path)
+                self.assertEqual(get_filesystem_type(
+                    check_path.lower())[0], 'NTFS')
+                self.assertEqual(get_filesystem_type(
+                    check_path.upper())[0], 'NTFS')
+        elif os.name == 'posix':
+            for check_path in (home, '/'):
+                detected_fs = get_filesystem_type(check_path)[0]
+                self.assertIn(detected_fs, ['btrfs', 'ext4', 'ext3', 'squashfs', 'unknown'],
+                              f"Unexpected file system type for {check_path}: {detected_fs}")
+
     def test_getsize(self):
         """Unit test for method getsize()"""
         dirname = self.mkdtemp(prefix='bleachbit-test-getsize')
 
         def test_getsize_helper(fname):
             filename = self.write_file(os.path.join(
-                dirname, fname), "abcdefghij" * 12345)
+                dirname, fname), b"abcdefghij" * 12345)
 
             if 'nt' == os.name:
                 self.assertEqual(getsize(filename), 10 * 12345)
@@ -579,8 +745,9 @@ class FileUtilitiesTestCase(common.BleachbitTestCase):
                     counter += 1
                 self.assertEqual(counter, 1)
             if 'posix' == os.name:
-                output = subprocess.Popen(
-                    ["du", "-h", filename], stdout=subprocess.PIPE).communicate()[0]
+                encoding = sys.getdefaultencoding()
+                output = str(subprocess.Popen(
+                    ["du", "-h", filename], stdout=subprocess.PIPE).communicate()[0], encoding=encoding)
                 output = output.replace("\n", "")
                 du_size = output.split("\t")[0] + "B"
                 print("output = '%s', size='%s'" % (output, du_size))
@@ -594,13 +761,13 @@ class FileUtilitiesTestCase(common.BleachbitTestCase):
         test_getsize_helper('bleachbit-test-regular')
 
         # special characters
-        test_getsize_helper(u'bleachbit-test-special-characters-∺ ∯')
+        test_getsize_helper('bleachbit-test-special-characters-∺ ∯')
 
         # em-dash (LP1454030)
-        test_getsize_helper(u'bleachbit-test-em-dash-\u2014')
+        test_getsize_helper('bleachbit-test-em-dash-\u2014')
 
         # long
-        test_getsize_helper(u'bleachbit-test-long' + 'x' * 200)
+        test_getsize_helper('bleachbit-test-long' + 'x' * 200)
 
         # delete the empty directory
         delete(dirname)
@@ -611,7 +778,7 @@ class FileUtilitiesTestCase(common.BleachbitTestCase):
 
         # create a symlink
         filename = self.write_file(
-            'bleachbit-test-symlink', 'abcdefghij' * 12345)
+            'bleachbit-test-symlink', b'abcdefghij' * 12345)
         linkname = os.path.join(self.tempdir, 'bleachbitsymlinktest')
         if os.path.lexists(linkname):
             delete(linkname)
@@ -659,22 +826,22 @@ class FileUtilitiesTestCase(common.BleachbitTestCase):
         valid = {'1kB': 1000,
                  '1.1MB': 1100000,
                  '12B': 12,
-                 '1.0M': 1000*1000,
+                 '1.0M': 1000 * 1000,
                  '1TB': 1000**4,
                  '1000': 1000}
         for test, result in valid.items():
             self.assertEqual(human_to_bytes(test), result)
 
-        self.assertEqual(human_to_bytes('1 MB', 'du'), 1024*1024)
+        self.assertEqual(human_to_bytes('1 MB', 'du'), 1024 * 1024)
 
     def test_listdir(self):
         """Unit test for listdir()"""
         if 'posix' == os.name:
             dir1 = '/bin'
-            dir2 = expanduser('/sbin')
-        if 'nt' == os.name:
-            dir1 = expandvars(r'%windir%\fonts')
-            dir2 = expandvars(r'%windir%\logs')
+            dir2 = os.path.expanduser('/sbin')
+        elif 'nt' == os.name:
+            dir1 = os.path.expandvars(r'%windir%\fonts')
+            dir2 = os.path.expandvars(r'%windir%\logs')
         # If these directories do not exist, the test results are not valid.
         self.assertExists(dir1)
         self.assertExists(dir2)
@@ -695,13 +862,40 @@ class FileUtilitiesTestCase(common.BleachbitTestCase):
         for pathname in paths12:
             self.assertLExists(pathname)
 
+    def test_open_files_lsof(self):
+        self.assertEqual(list(open_files_lsof(
+            lambda: 'n/bar/foo\nn/foo/bar\nnoise')), ['/bar/foo', '/foo/bar'])
+
+    @common.skipIfWindows
+    def test_OpenFiles(self):
+        """Unit test for class OpenFiles"""
+
+        filename = os.path.join(self.tempdir, 'bleachbit-test-open-files')
+        f = open(filename, 'w')
+        openfiles = OpenFiles()
+        self.assertTrue(openfiles.is_open(filename),
+                        "Expected is_open(%s) to return True)\n"
+                        "openfiles.last_scan_time (ago)=%s\n"
+                        "openfiles.files=%s" %
+                        (filename,
+                         time.time() - openfiles.last_scan_time,
+                         openfiles.files))
+
+        f.close()
+        openfiles.scan()
+        self.assertFalse(openfiles.is_open(filename))
+
+        os.unlink(filename)
+        openfiles.scan()
+        self.assertFalse(openfiles.is_open(filename))
+
     def test_same_partition(self):
         """Unit test for same_partition()"""
-        home = expanduser('~')
+        home = os.path.expanduser('~')
         self.assertTrue(same_partition(home, home))
         if 'posix' == os.name:
             self.assertFalse(same_partition(home, '/dev'))
-        if 'nt' == os.name:
+        elif 'nt' == os.name:
             home_drive = os.path.splitdrive(home)[0]
             from bleachbit.Windows import get_fixed_drives
             for drive in get_fixed_drives():
@@ -715,12 +909,12 @@ class FileUtilitiesTestCase(common.BleachbitTestCase):
 
         # Unix-style
         uri_u = ['file:///usr/bin/bleachbit']
-        path_u = [os.path.normpath(u'/usr/bin/bleachbit'), ]
+        path_u = [os.path.normpath('/usr/bin/bleachbit'), ]
         self.assertEqual(uris_to_paths(uri_u), path_u)
 
         # Windows
         uri_w = [r'file:///C:/program%20files/bleachbit.exe']
-        path_w = [os.path.normpath(ur'C:/program files/bleachbit.exe'), ]
+        path_w = [os.path.normpath(r'C:/program files/bleachbit.exe'), ]
         self.assertEqual(uris_to_paths(uri_w), path_w)
 
         # Multiple
@@ -730,6 +924,34 @@ class FileUtilitiesTestCase(common.BleachbitTestCase):
         uri_s = ['foo://bar']
         self.assertEqual(uris_to_paths(uri_u + uri_w + uri_s), path_u + path_w)
 
+    def test_vacuum_sqlite3(self):
+        """Unit test for method vacuum_sqlite3()"""
+        import sqlite3
+
+        path = os.path.join(self.tempdir, 'bleachbit.tmp.sqlite3')
+        conn = sqlite3.connect(path)
+        conn.execute('create table numbers (number)')
+        conn.commit()
+        empty_size = getsize(path)
+
+        def number_generator():
+            for x in range(1, 10000):
+                yield (x, )
+        conn.executemany(
+            'insert into numbers (number) values ( ? ) ', number_generator())
+        conn.commit()
+        self.assertLess(empty_size, getsize(path))
+        conn.execute('delete from numbers')
+        conn.commit()
+        conn.close()
+
+        vacuum_sqlite3(path)
+        self.assertEqual(empty_size, getsize(path))
+
+        gc_collect()
+        delete(path)
+        self.assertNotExists(path)
+
     def test_whitelisted(self):
         """Unit test for whitelisted()"""
         # setup
@@ -821,8 +1043,8 @@ class FileUtilitiesTestCase(common.BleachbitTestCase):
         d = '/usr/bin'
         whitelist = [('file', '/home/foo'), ('folder', '/home/folder')]
         if 'nt' == os.name:
-            d = expandvars('%windir%\system32')
-            whitelist = [('file', r'c:\\filename'), ('folder', r'c:\\folder')]
+            d = os.path.expandvars(r'%windir%\system32')
+            whitelist = [('file', r'c:\filename'), ('folder', r'c:\\folder')]
         reps = 20
         paths = [p for p in children_in_directory(d, True)]
         paths = paths[:1000]  # truncate
@@ -831,7 +1053,7 @@ class FileUtilitiesTestCase(common.BleachbitTestCase):
         options.set_whitelist_paths(whitelist)
 
         t0 = time.time()
-        for i in xrange(0, reps):
+        for _i in range(0, reps):
             for p in paths:
                 _ = whitelisted(p)
         t1 = time.time()
@@ -844,7 +1066,8 @@ class FileUtilitiesTestCase(common.BleachbitTestCase):
         """Unit test for wipe_delete()"""
 
         # create test file
-        filename = self.write_file('bleachbit-test-wipe', 'abcdefghij' * 12345)
+        filename = self.write_file(
+            'bleachbit-test-wipe', b'abcdefghij' * 12345)
 
         # wipe it
         wipe_contents(filename)
@@ -853,9 +1076,9 @@ class FileUtilitiesTestCase(common.BleachbitTestCase):
         f = open(filename, 'rb')
         while True:
             byte = f.read(1)
-            if "" == byte:
+            if b"" == byte:
                 break
-            self.assertEqual(byte, chr(0))
+            self.assertEqual(byte, 0)
         f.close()
 
         # clean up
@@ -921,62 +1144,20 @@ class FileUtilitiesTestCase(common.BleachbitTestCase):
     def test_wipe_path(self):
         """Unit test for wipe_path()"""
 
-        for ret in wipe_path(self.tempdir):
+        for _ret in wipe_path(self.tempdir):
             # no idle handler
             pass
 
-    def test_vacuum_sqlite3(self):
-        """Unit test for method vacuum_sqlite3()"""
-        import sqlite3
+    def test_wipe_path_fast(self):
+        """Unit test for wipe_path() with fast mode
 
-        path = os.path.abspath('bleachbit.tmp.sqlite3')
-        if os.path.lexists(path):
-            delete(path)
-
-        conn = sqlite3.connect(path)
-        conn.execute('create table numbers (number)')
-        conn.commit()
-        empty_size = getsize(path)
-
-        def number_generator():
-            for x in range(1, 10000):
-                yield (x, )
-        conn.executemany(
-            'insert into numbers (number) values ( ? ) ', number_generator())
-        conn.commit()
-        self.assertLess(empty_size, getsize(path))
-        conn.execute('delete from numbers')
-        conn.commit()
-        conn.close()
-
-        vacuum_sqlite3(path)
-        self.assertEqual(empty_size, getsize(path))
-
-        delete(path)
-
-    @common.skipIfWindows
-    def test_OpenFiles(self):
-        """Unit test for class OpenFiles"""
-
-        filename = os.path.join(self.tempdir, 'bleachbit-test-open-files')
-        f = open(filename, 'w')
-        openfiles = OpenFiles()
-        self.assertTrue(openfiles.is_open(filename),
-                        "Expected is_open(%s) to return True)\n"
-                        "openfiles.last_scan_time (ago)=%s\n"
-                        "openfiles.files=%s" %
-                        (filename,
-                         time.time() - openfiles.last_scan_time,
-                         openfiles.files))
-
-        f.close()
-        openfiles.scan()
-        self.assertFalse(openfiles.is_open(filename))
-
-        os.unlink(filename)
-        openfiles.scan()
-        self.assertFalse(openfiles.is_open(filename))
-
-    def test_open_files_lsof(self):
-        self.assertEqual(list(open_files_lsof(
-            lambda: 'n/bar/foo\nn/foo/bar\nnoise')), ['/bar/foo', '/foo/bar'])
+        This test runs three iterations of the generator
+        and then aborts. Each iteration takes a little more
+        than two seconds.
+        """
+        counter = 0
+        for _i in wipe_path(self.tempdir, True):
+            counter += 1
+            if counter >= 3:
+                break
+        self.assertGreater(counter, 0)
